VERSION 5.00
Begin VB.UserControl TabDock 
   Appearance      =   0  'Flat
   BackColor       =   &H80000005&
   CanGetFocus     =   0   'False
   ClientHeight    =   528
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   588
   ClipControls    =   0   'False
   EditAtDesignTime=   -1  'True
   InvisibleAtRuntime=   -1  'True
   Picture         =   "TabDock.ctx":0000
   ScaleHeight     =   44
   ScaleMode       =   3  'Pixel
   ScaleWidth      =   49
   ToolboxBitmap   =   "TabDock.ctx":0842
End
Attribute VB_Name = "TabDock"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' ******************************************************************************
' Control:           TabDock.ctl
' Description:       Main part of the engine, This is the user control
' Created by:        Marclei V Silva
' Machine:           ZEUS
' Date-Time:         09/05/20003:13:22
' Last modification: last_modification_info_here
'
'   Date            Author      Description
'   10/05/2000      MVS         Docking docking/moving/resizing
'                               was improved
' ******************************************************************************
Option Explicit

Private Const BF_SOFT = &H1000 ' For softer buttons
Private Const BDR_RAISEDOUTER = &H1

' This array will hold the panels corresponding
' each corner of the screen top, left, bottom and right
Private m_Panels(1 To 4) As TabDockHost
Private m_cDockedForms As DockedForms
Private m_bInitializing As Boolean

' Events Held by this control
Public Event FormDocked(DockedForm As DockForm)
Public Event FormUnDocked(DockedForm As DockForm)

'Default Property Values:
Const m_def_PanelHeight = 1000
Const m_def_PanelWidth = 2100

'Property Variables:
Dim m_PanelHeight As Integer
Dim m_PanelWidth As Integer

Property Get Parent() As Object
    Parent = Extender.Parent
End Property

Property Get Panels() As Variant
    Panels = m_Panels
End Property

Property Get DockedForms() As DockedForms
    Set DockedForms = m_cDockedForms
End Property

Private Sub UserControl_Initialize()
    m_bInitializing = True
    Set m_cDockedForms = New DockedForms
End Sub

Private Sub UserControl_Terminate()
    Set m_cDockedForms = Nothing
End Sub

Private Sub UserControl_Paint()
    Dim edge As RECT                                ' Rectangle edge of control
    edge.Left = 0                                   ' Set rect edges to outer
    edge.Top = 0                                    ' most position in pixels
    edge.Bottom = ScaleHeight
    edge.Right = ScaleWidth
    DrawEdge hdc, edge, BDR_RAISEDOUTER, BF_RECT Or BF_SOFT ' Draw Edge...
End Sub

Private Sub UserControl_Resize()
    UserControl.Width = 32 * Screen.TwipsPerPixelX
    UserControl.Height = 32 * Screen.TwipsPerPixelY
End Sub

' Adds forms to the main engine
Public Function Add(Item As Object, Optional State As eDockedState = undocked, _
    Optional Align As Integer = 3, _
    Optional Key As String, _
    Optional AllowDockTop As Boolean, _
    Optional AllowDockLeft As Boolean, _
    Optional AllowDockBottom As Boolean, _
    Optional AllowDockRight As Boolean, _
    Optional AllowFloat As Boolean) As DockForm
    
    ' if panels were not created then create them now
    If m_bInitializing Then
        LoadPanels
        m_bInitializing = False
    End If
    
    ' change the border to fixed toolwindow
    SetWindowLong Item.hwnd, GWL_EXSTYLE, WS_EX_TOOLWINDOW
    Set Add = m_cDockedForms.Add( _
        Item.hwnd, m_Panels(Align), _
         AllowDockTop, AllowDockLeft, AllowDockBottom, AllowDockRight, _
         AllowFloat, State, _
        Item.Width, Item.Height, Item.Left, Item.Top, Key _
    )
End Function

' Load the panels for docking engine
' CAUTION: If your project is ignoring not used ActiveX
' Thid function may report a error or not function,
' once the docktabhost
' is not used until reaching this point.
Private Sub LoadPanels()
    Dim i As Integer
    Dim rc As RECT
    
    On Error GoTo process_error
    ' get client rect
    GetClientRect Extender.Parent.hwnd, rc
    ' create the panels
    For i = 1 To 4
        Set m_Panels(i) = LoadControl(Extender.Parent, "DockedWindows.TabDockHost", "Panel" & CStr(i))
        m_Panels(i).Align = i
        m_Panels(i).Visible = False        ' Panels are initially invisible till we dock some form
        Set m_Panels(i).Parent = Me        ' set ME as parent of the Panel
    Next
    Exit Sub
process_error:
End Sub

' This code was extract from
' FreeVBCode.com (http://www.freevbcode.com)
Private Function LoadControl(oForm As Object, CtlType As String, CtlName As String) As Object
    Dim oCtl As Object
    On Error Resume Next
    If IsObject(oForm.Controls) Then
        Set oCtl = oForm.Controls.Add(CtlType, CtlName)
        If Not oCtl Is Nothing Then
            Set LoadControl = oCtl
            Set oCtl = Nothing
        End If
    End If
End Function

Public Sub Show()
    Dim i As Integer
    LockWindowUpdate Extender.Parent.hwnd
    For i = 1 To m_cDockedForms.Count
        If m_cDockedForms(i).State = docked Then
            m_cDockedForms(i).DockWindowHost.Dock m_cDockedForms(i).DockWindow, m_cDockedForms(i).DockWindowHost.Align
        Else
            m_cDockedForms(i).DockWindowHost.UnDockShow m_cDockedForms(i).DockWindow
        End If
    Next
    For i = 1 To 4
        m_Panels(i).Visible = True
        Select Case i
        Case vbAlignLeft
            m_Panels(i).Left = 0
            m_Panels(i).Width = m_PanelWidth
        Case vbAlignRight
            'm_Panels(i).Left = 0
            m_Panels(i).Width = m_PanelWidth
        Case vbAlignTop
            m_Panels(i).Top = 1000
            m_Panels(i).Height = m_PanelHeight
        Case vbAlignBottom
            m_Panels(i).Top = 0
            m_Panels(i).Height = m_PanelHeight
        End Select
        'm_Panels(i).Align = i              ' align to the panel
        m_Panels(i).Refresh                ' refresh TabDockHost
        m_Panels(i).Visible = m_Panels(i).Controls.Count > 0
    Next
    LockWindowUpdate 0
End Sub

Friend Sub Dock(hWnda As Long)
    Dim idx As Integer
    For idx = 1 To m_cDockedForms.Count
        If m_cDockedForms(idx).DockWindow = hWnda Then
            Exit For
        End If
    Next
    If idx <= m_cDockedForms.Count Then
        RaiseEvent FormDocked(m_cDockedForms(idx))
    End If
End Sub

Friend Sub UnDock(hWnda As Long)
    Dim idx As Integer
    For idx = 1 To m_cDockedForms.Count
        If m_cDockedForms(idx).DockWindow = hWnda Then
            Exit For
        End If
    Next
    If idx <= m_cDockedForms.Count Then
        RaiseEvent FormUnDocked(m_cDockedForms(idx))
    End If
End Sub

'WARNING! DO NOT REMOVE OR MODIFY THE FOLLOWING COMMENTED LINES!
'MemberInfo=7,0,0,2100
Public Property Get PanelHeight() As Integer
Attribute PanelHeight.VB_Description = "Initial height of the Top and Bottom panels of TabDock engine"
    PanelHeight = m_PanelHeight
End Property

Public Property Let PanelHeight(ByVal New_PanelHeight As Integer)
    m_PanelHeight = New_PanelHeight
    PropertyChanged "PanelHeight"
End Property

'WARNING! DO NOT REMOVE OR MODIFY THE FOLLOWING COMMENTED LINES!
'MemberInfo=7,0,0,1000
Public Property Get PanelWidth() As Integer
Attribute PanelWidth.VB_Description = "Initial width of the left and right panels of TabDock engine"
    PanelWidth = m_PanelWidth
End Property

Public Property Let PanelWidth(ByVal New_PanelWidth As Integer)
    m_PanelWidth = New_PanelWidth
    PropertyChanged "PanelWidth"
End Property

'Initialize Properties for User Control
Private Sub UserControl_InitProperties()
    m_PanelHeight = m_def_PanelHeight
    m_PanelWidth = m_def_PanelWidth
End Sub

'Load property values from storage
Private Sub UserControl_ReadProperties(PropBag As PropertyBag)

    m_PanelHeight = PropBag.ReadProperty("PanelHeight", m_def_PanelHeight)
    m_PanelWidth = PropBag.ReadProperty("PanelWidth", m_def_PanelWidth)
End Sub

'Write property values to storage
Private Sub UserControl_WriteProperties(PropBag As PropertyBag)

    Call PropBag.WriteProperty("PanelHeight", m_PanelHeight, m_def_PanelHeight)
    Call PropBag.WriteProperty("PanelWidth", m_PanelWidth, m_def_PanelWidth)
End Sub

